"""Module for mapping ISO 639-3 to Glottolog codes and language names.

ISO 639-3 us an international standard for language codes with an
aim to cover all known natural languages. The extended language
coverage was based primarily on the language codes published by
SIL International, which is now the registration authority for
ISO 639-3. About: https://iso639-3.sil.org/

Glottolog is a project run by the Max Planck Institute for the
Science of Human History. The website contains codes for languages
as well as reconstructions of language families. About: http://glottolog.org/.


Data of Glottolog 4.0 is published under the following license:
https://creativecommons.org/licenses/by/4.0/

Haspelmath, Martin & Forkel, Robert & Hammarström, Harald. 2019.
Glottolog 4.0.
Jena: Max Planck Institute for the Science of Human History.
(Available online at http://glottolog.org, Accessed on 2019-06-28.)
"""

import csv
import os
from collections import defaultdict, namedtuple

# Attributes come from Glottolog except type, which is from
# ISO 639-3 ("A": Ancient, "H": Historical)
Language = namedtuple(
    "Language", ["family_id", "parent_id", "name", "level", "iso639P3code", "type"]
)

# These are the Glottolog language descriptions which ISO calls Historical or Ancient.
# This was produced by the script at the bottom of this file.
ISO_TO_GLOTTO = {
    "aghw1237": Language(
        family_id="nakh1245",
        parent_id="udia1235",
        name="Aghwan",
        level="language",
        iso639P3code="xag",
        type="a",
    ),
    "akka1240": Language(
        family_id="afro1255",
        parent_id="east2678",
        name="Akkadian",
        level="language",
        iso639P3code="akk",
        type="a",
    ),
    "anci1242": Language(
        family_id="indo1319",
        parent_id="east2798",
        name="Ancient Greek",
        level="language",
        iso639P3code="grc",
        type="h",
    ),
    "anci1244": Language(
        family_id="afro1255",
        parent_id="hebr1246",
        name="Ancient Hebrew",
        level="language",
        iso639P3code="hbo",
        type="h",
    ),
    "anci1245": Language(
        family_id="afro1255",
        parent_id="arab1394",
        name="Ancient North Arabian",
        level="language",
        iso639P3code="xna",
        type="a",
    ),
    "anci1248": Language(
        family_id="uncl1493",
        parent_id="uncl1493",
        name="Ancient Ligurian",
        level="language",
        iso639P3code="xlg",
        type="a",
    ),
    "anda1287": Language(
        family_id="afro1255",
        parent_id="moro1295",
        name="Andalusian Arabic",
        level="language",
        iso639P3code="xaa",
        type="h",
    ),
    "angl1258": Language(
        family_id="indo1319",
        parent_id="stan1290",
        name="Anglo-Norman",
        level="dialect",
        iso639P3code="xno",
        type="h",
    ),
    "aves1237": Language(
        family_id="indo1319",
        parent_id="iran1269",
        name="Avestan",
        level="language",
        iso639P3code="ave",
        type="a",
    ),
    "bact1239": Language(
        family_id="indo1319",
        parent_id="cent2318",
        name="Bactrian",
        level="language",
        iso639P3code="xbc",
        type="a",
    ),
    "bolg1250": Language(
        family_id="turk1311",
        parent_id="bolg1249",
        name="Bolgarian",
        level="language",
        iso639P3code="xbo",
        type="h",
    ),
    "burm1262": Language(
        family_id="sino1245",
        parent_id="uncl1508",
        name="Burma Pyu",
        level="language",
        iso639P3code="pyx",
        type="a",
    ),
    "camu1235": Language(
        family_id="uncl1493",
        parent_id="uncl1493",
        name="Camunic",
        level="language",
        iso639P3code="xcc",
        type="a",
    ),
    "cari1274": Language(
        family_id="indo1319",
        parent_id="luvi1234",
        name="Carian",
        level="language",
        iso639P3code="xcr",
        type="a",
    ),
    "celt1247": Language(
        family_id="indo1319",
        parent_id="celt1248",
        name="Celtiberian",
        level="language",
        iso639P3code="xce",
        type="a",
    ),
    "chur1257": Language(
        family_id="indo1319",
        parent_id="east2269",
        name="Church Slavic",
        level="language",
        iso639P3code="chu",
        type="a",
    ),
    "cisa1237": Language(
        family_id="indo1319",
        parent_id="cisa1238",
        name="Cisalpine Gaulish",
        level="language",
        iso639P3code="xcg",
        type="a",
    ),
    "clas1249": Language(
        family_id="indo1319",
        parent_id="arme1241",
        name="Classical Armenian",
        level="language",
        iso639P3code="xcl",
        type="h",
    ),
    "clas1250": Language(
        family_id="utoa1244",
        parent_id="cent2258",
        name="Classical Nahuatl",
        level="language",
        iso639P3code="nci",
        type="h",
    ),
    "clas1251": Language(
        family_id="quec1387",
        parent_id="quec1389",
        name="Classical Quechua",
        level="language",
        iso639P3code="qwc",
        type="h",
    ),
    "clas1252": Language(
        family_id="afro1255",
        parent_id="east2680",
        name="Classical Syriac",
        level="language",
        iso639P3code="syc",
        type="h",
    ),
    "clas1253": Language(
        family_id="afro1255",
        parent_id="mand1468",
        name="Classical Mandaic",
        level="language",
        iso639P3code="myz",
        type="h",
    ),
    "clas1254": Language(
        family_id="sino1245",
        parent_id="oldm1245",
        name="Classical Tibetan",
        level="language",
        iso639P3code="xct",
        type="h",
    ),
    "cune1239": Language(
        family_id="indo1319",
        parent_id="luvi1235",
        name="Cuneiform Luwian",
        level="language",
        iso639P3code="xlu",
        type="a",
    ),
    "daci1234": Language(
        family_id="indo1319",
        parent_id="indo1319",
        name="Dacian",
        level="language",
        iso639P3code="xdc",
        type="a",
    ),
    "ebla1238": Language(
        family_id="afro1255",
        parent_id="east2678",
        name="Eblaite",
        level="language",
        iso639P3code="xeb",
        type="a",
    ),
    "edom1234": Language(
        family_id="book1242",
        parent_id="book1242",
        name="Edomite",
        level="language",
        iso639P3code="xdm",
        type="a",
    ),
    "egyp1246": Language(
        family_id="afro1255",
        parent_id="egyp1245",
        name="Egyptian (Ancient)",
        level="language",
        iso639P3code="egy",
        type="a",
    ),
    "elam1244": Language(
        family_id="",
        parent_id="",
        name="Elamite",
        level="language",
        iso639P3code="elx",
        type="a",
    ),
    "elym1237": Language(
        family_id="uncl1493",
        parent_id="uncl1493",
        name="Elymian",
        level="language",
        iso639P3code="xly",
        type="a",
    ),
    "epig1241": Language(
        family_id="maya1287",
        parent_id="chol1287",
        name="Epigraphic Mayan",
        level="language",
        iso639P3code="emy",
        type="a",
    ),
    "eteo1236": Language(
        family_id="uncl1493",
        parent_id="uncl1493",
        name="Eteocretan",
        level="language",
        iso639P3code="ecr",
        type="a",
    ),
    "eteo1240": Language(
        family_id="uncl1493",
        parent_id="uncl1493",
        name="Eteocypriot",
        level="language",
        iso639P3code="ecy",
        type="a",
    ),
    "etru1241": Language(
        family_id="",
        parent_id="",
        name="Etruscan",
        level="language",
        iso639P3code="ett",
        type="a",
    ),
    "fali1291": Language(
        family_id="indo1319",
        parent_id="lati1262",
        name="Faliscan",
        level="language",
        iso639P3code="xfa",
        type="a",
    ),
    "fran1264": Language(
        family_id="indo1319",
        parent_id="oldd1237",
        name="Old Frankish",
        level="dialect",
        iso639P3code="frk",
        type="h",
    ),
    "gala1252": Language(
        family_id="indo1319",
        parent_id="nucl1715",
        name="Galatian",
        level="language",
        iso639P3code="xga",
        type="a",
    ),
    "geez1241": Language(
        family_id="afro1255",
        parent_id="ethi1244",
        name="Geez",
        level="language",
        iso639P3code="gez",
        type="a",
    ),
    "goth1244": Language(
        family_id="indo1319",
        parent_id="east2805",
        name="Gothic",
        level="language",
        iso639P3code="got",
        type="a",
    ),
    "hadr1235": Language(
        family_id="afro1255",
        parent_id="sayh1236",
        name="Hadrami",
        level="language",
        iso639P3code="xhd",
        type="a",
    ),
    "hara1272": Language(
        family_id="unat1236",
        parent_id="unat1236",
        name="Harappan",
        level="language",
        iso639P3code="xiv",
        type="a",
    ),
    "hatt1246": Language(
        family_id="",
        parent_id="",
        name="Hattic",
        level="language",
        iso639P3code="xht",
        type="a",
    ),
    "hemi1234": Language(
        family_id="indo1319",
        parent_id="osca1245",
        name="Hernican",
        level="dialect",
        iso639P3code="xhr",
        type="a",
    ),
    "hibe1235": Language(
        family_id="indo1319",
        parent_id="oldi1245",
        name="Hiberno-Scottish Gaelic",
        level="dialect",
        iso639P3code="ghc",
        type="h",
    ),
    "hier1240": Language(
        family_id="indo1319",
        parent_id="luvi1235",
        name="Hieroglyphic Luwian",
        level="language",
        iso639P3code="hlu",
        type="a",
    ),
    "hitt1242": Language(
        family_id="indo1319",
        parent_id="anat1257",
        name="Hittite",
        level="language",
        iso639P3code="hit",
        type="a",
    ),
    "hurr1240": Language(
        family_id="hurr1239",
        parent_id="hurr1239",
        name="Hurrian",
        level="language",
        iso639P3code="xhu",
        type="a",
    ),
    "iber1250": Language(
        family_id="",
        parent_id="",
        name="Iberian",
        level="language",
        iso639P3code="xib",
        type="a",
    ),
    "illy1234": Language(
        family_id="uncl1493",
        parent_id="uncl1493",
        name="Illyrian",
        level="language",
        iso639P3code="xil",
        type="a",
    ),
    "juti1236": Language(
        family_id="indo1319",
        parent_id="sout3248",
        name="Jutish",
        level="language",
        iso639P3code="jut",
        type="h",
    ),
    "kajk1237": Language(
        family_id="indo1319",
        parent_id="west2804",
        name="Kajkavian",
        level="language",
        iso639P3code="kjv",
        type="h",
    ),
    "kawi1241": Language(
        family_id="aust1307",
        parent_id="java1253",
        name="Kawi",
        level="language",
        iso639P3code="kaw",
        type="a",
    ),
    "khot1251": Language(
        family_id="indo1319",
        parent_id="saka1298",
        name="Khotanese",
        level="language",
        iso639P3code="kho",
        type="a",
    ),
    "khwa1238": Language(
        family_id="indo1319",
        parent_id="sogd1248",
        name="Khwarezmian",
        level="language",
        iso639P3code="xco",
        type="a",
    ),
    "kita1247": Language(
        family_id="mong1329",
        parent_id="unun9946",
        name="Kitan",
        level="language",
        iso639P3code="zkt",
        type="h",
    ),
    "kogu1234": Language(
        family_id="uncl1493",
        parent_id="uncl1493",
        name="Koguryo",
        level="language",
        iso639P3code="zkg",
        type="a",
    ),
    "lati1261": Language(
        family_id="indo1319",
        parent_id="impe1234",
        name="Latin",
        level="language",
        iso639P3code="lat",
        type="a",
    ),
    "lemn1237": Language(
        family_id="uncl1493",
        parent_id="uncl1493",
        name="Lemnian",
        level="language",
        iso639P3code="xle",
        type="a",
    ),
    "lepo1240": Language(
        family_id="indo1319",
        parent_id="cisa1238",
        name="Lepontic",
        level="language",
        iso639P3code="xlp",
        type="a",
    ),
    "lite1248": Language(
        family_id="sino1245",
        parent_id="clas1255",
        name="Literary Chinese",
        level="language",
        iso639P3code="lzh",
        type="h",
    ),
    "lusi1235": Language(
        family_id="indo1319",
        parent_id="indo1319",
        name="Lusitanian",
        level="language",
        iso639P3code="xls",
        type="a",
    ),
    "lyci1241": Language(
        family_id="indo1319",
        parent_id="lyci1242",
        name="Lycian A",
        level="language",
        iso639P3code="xlc",
        type="a",
    ),
    "lydi1241": Language(
        family_id="indo1319",
        parent_id="unun9888",
        name="Lydian",
        level="language",
        iso639P3code="xld",
        type="a",
    ),
    "maha1305": Language(
        family_id="indo1319",
        parent_id="indo1325",
        name="Maharastri Prakrit",
        level="language",
        iso639P3code="pmh",
        type="h",
    ),
    "mero1237": Language(
        family_id="",
        parent_id="",
        name="Meroitic",
        level="language",
        iso639P3code="xmr",
        type="a",
    ),
    "mess1244": Language(
        family_id="indo1319",
        parent_id="indo1319",
        name="Messapic",
        level="language",
        iso639P3code="cms",
        type="a",
    ),
    "midd1316": Language(
        family_id="indo1319",
        parent_id="stan1290",
        name="Middle French",
        level="dialect",
        iso639P3code="frm",
        type="h",
    ),
    "midd1317": Language(
        family_id="indo1319",
        parent_id="merc1242",
        name="Middle English",
        level="language",
        iso639P3code="enm",
        type="h",
    ),
    "midd1318": Language(
        family_id="indo1319",
        parent_id="midd1345",
        name="Middle Low German",
        level="language",
        iso639P3code="gml",
        type="h",
    ),
    "midd1321": Language(
        family_id="indo1319",
        parent_id="midd1347",
        name="Middle Dutch",
        level="language",
        iso639P3code="dum",
        type="h",
    ),
    "midd1343": Language(
        family_id="indo1319",
        parent_id="midd1349",
        name="Middle High German",
        level="language",
        iso639P3code="gmh",
        type="h",
    ),
    "midd1344": Language(
        family_id="sino1245",
        parent_id="midd1354",
        name="Middle Chinese",
        level="language",
        iso639P3code="ltc",
        type="h",
    ),
    "midd1351": Language(
        family_id="mong1329",
        parent_id="mong1329",
        name="Middle Mongol",
        level="language",
        iso639P3code="xng",
        type="h",
    ),
    "midd1360": Language(
        family_id="indo1319",
        parent_id="oldi1245",
        name="Middle Irish (10-12th century)",
        level="dialect",
        iso639P3code="mga",
        type="h",
    ),
    "mily1238": Language(
        family_id="indo1319",
        parent_id="lyci1242",
        name="Milyan",
        level="language",
        iso639P3code="imy",
        type="a",
    ),
    "mina1279": Language(
        family_id="afro1255",
        parent_id="sayh1236",
        name="Minaean",
        level="language",
        iso639P3code="inm",
        type="a",
    ),
    "mino1236": Language(
        family_id="uncl1493",
        parent_id="uncl1493",
        name="Minoan",
        level="language",
        iso639P3code="omn",
        type="a",
    ),
    "moab1234": Language(
        family_id="book1242",
        parent_id="book1242",
        name="Moabite",
        level="language",
        iso639P3code="obm",
        type="a",
    ),
    "moza1249": Language(
        family_id="indo1319",
        parent_id="unsh1234",
        name="Mozarabic",
        level="language",
        iso639P3code="mxi",
        type="h",
    ),
    "myce1241": Language(
        family_id="indo1319",
        parent_id="east2798",
        name="Mycenaean Greek",
        level="language",
        iso639P3code="gmy",
        type="a",
    ),
    "mysi1239": Language(
        family_id="indo1319",
        parent_id="phry1239",
        name="Mysian",
        level="dialect",
        iso639P3code="yms",
        type="a",
    ),
    "nori1240": Language(
        family_id="indo1319",
        parent_id="unun9887",
        name="Noric",
        level="language",
        iso639P3code="nrc",
        type="a",
    ),
    "nort1401": Language(
        family_id="uncl1493",
        parent_id="uncl1493",
        name="North Picene",
        level="language",
        iso639P3code="nrp",
        type="a",
    ),
    "numi1241": Language(
        family_id="uncl1493",
        parent_id="uncl1493",
        name="Numidian",
        level="language",
        iso639P3code="nxm",
        type="a",
    ),
    "olda1245": Language(
        family_id="afro1255",
        parent_id="aram1259",
        name="Old Aramaic (up to 700 BCE)",
        level="language",
        iso639P3code="oar",
        type="a",
    ),
    "oldb1235": Language(
        family_id="sino1245",
        parent_id="oldm1246",
        name="Old Burmese",
        level="language",
        iso639P3code="obr",
        type="h",
    ),
    "oldc1244": Language(
        family_id="sino1245",
        parent_id="sini1245",
        name="Old Chinese",
        level="language",
        iso639P3code="och",
        type="a",
    ),
    "oldc1252": Language(
        family_id="indo1319",
        parent_id="corn1251",
        name="Old Cornish",
        level="dialect",
        iso639P3code="oco",
        type="h",
    ),
    "oldd1237": Language(
        family_id="indo1319",
        parent_id="macr1270",
        name="Old Dutch-Old Frankish",
        level="language",
        iso639P3code="odt",
        type="h",
    ),
    "olde1238": Language(
        family_id="indo1319",
        parent_id="angl1265",
        name="Old English (ca. 450-1100)",
        level="language",
        iso639P3code="ang",
        type="h",
    ),
    "oldf1239": Language(
        family_id="indo1319",
        parent_id="oila1234",
        name="Old French (842-ca. 1400)",
        level="language",
        iso639P3code="fro",
        type="h",
    ),
    "oldf1241": Language(
        family_id="indo1319",
        parent_id="fris1239",
        name="Old Frisian",
        level="language",
        iso639P3code="ofs",
        type="h",
    ),
    "oldg1234": Language(
        family_id="kart1248",
        parent_id="geor1253",
        name="Old Georgian",
        level="language",
        iso639P3code="oge",
        type="h",
    ),
    "oldh1241": Language(
        family_id="indo1319",
        parent_id="high1286",
        name="Old High German (ca. 750-1050)",
        level="language",
        iso639P3code="goh",
        type="h",
    ),
    "oldi1245": Language(
        family_id="indo1319",
        parent_id="goid1240",
        name="Early Irish",
        level="language",
        iso639P3code="sga",
        type="h",
    ),
    "oldj1239": Language(
        family_id="japo1237",
        parent_id="japa1256",
        name="Old Japanese",
        level="language",
        iso639P3code="ojp",
        type="h",
    ),
    "oldl1240": Language(
        family_id="indo1319",
        parent_id="lith1251",
        name="Old Lithuanian",
        level="dialect",
        iso639P3code="olt",
        type="h",
    ),
    "oldm1242": Language(
        family_id="aust1305",
        parent_id="moni1258",
        name="Old Mon",
        level="language",
        iso639P3code="omx",
        type="h",
    ),
    "oldm1244": Language(
        family_id="indo1319",
        parent_id="mara1416",
        name="Old Marathi",
        level="language",
        iso639P3code="omr",
        type="h",
    ),
    "oldn1244": Language(
        family_id="indo1319",
        parent_id="west2805",
        name="Old Norse",
        level="language",
        iso639P3code="non",
        type="h",
    ),
    "oldn1245": Language(
        family_id="nubi1251",
        parent_id="nobi1239",
        name="Old Nubian",
        level="language",
        iso639P3code="onw",
        type="h",
    ),
    "oldo1234": Language(
        family_id="indo1319",
        parent_id="osse1245",
        name="Old Ossetic",
        level="language",
        iso639P3code="oos",
        type="a",
    ),
    "oldp1253": Language(
        family_id="indo1319",
        parent_id="occi1240",
        name="Old Provençal",
        level="language",
        iso639P3code="pro",
        type="h",
    ),
    "oldp1254": Language(
        family_id="indo1319",
        parent_id="sout3157",
        name="Old Persian (ca. 600-400 B.C.)",
        level="language",
        iso639P3code="peo",
        type="h",
    ),
    "oldr1238": Language(
        family_id="indo1319",
        parent_id="east1426",
        name="Old Russian",
        level="language",
        iso639P3code="orv",
        type="h",
    ),
    "olds1249": Language(
        family_id="indo1319",
        parent_id="cast1243",
        name="Old Spanish",
        level="language",
        iso639P3code="osp",
        type="h",
    ),
    "olds1250": Language(
        family_id="indo1319",
        parent_id="alts1234",
        name="Old Saxon",
        level="language",
        iso639P3code="osx",
        type="h",
    ),
    "oldt1247": Language(
        family_id="book1242",
        parent_id="book1242",
        name="Old Turkish",
        level="language",
        iso639P3code="otk",
        type="h",
    ),
    "oldt1248": Language(
        family_id="drav1251",
        parent_id="tami1299",
        name="Old Tamil",
        level="language",
        iso639P3code="oty",
        type="a",
    ),
    "oldu1238": Language(
        family_id="turk1311",
        parent_id="karl1243",
        name="Old Turkic",
        level="language",
        iso639P3code="oui",
        type="h",
    ),
    "oldw1239": Language(
        family_id="indo1319",
        parent_id="oldm1247",
        name="Old-Middle Welsh",
        level="language",
        iso639P3code="owl",
        type="h",
    ),
    "osca1245": Language(
        family_id="indo1319",
        parent_id="sabe1249",
        name="Oscan",
        level="language",
        iso639P3code="osc",
        type="a",
    ),
    "paek1234": Language(
        family_id="uncl1493",
        parent_id="uncl1493",
        name="Paekche",
        level="language",
        iso639P3code="pkc",
        type="a",
    ),
    "pahl1241": Language(
        family_id="indo1319",
        parent_id="midd1352",
        name="Pahlavi",
        level="language",
        iso639P3code="pal",
        type="a",
    ),
    "pala1331": Language(
        family_id="indo1319",
        parent_id="anat1257",
        name="Palaic",
        level="language",
        iso639P3code="plq",
        type="a",
    ),
    "pale1261": Language(
        family_id="afro1255",
        parent_id="west2815",
        name="Palestinian Jewish Aramaic",
        level="language",
        iso639P3code="jpa",
        type="h",
    ),
    "pali1273": Language(
        family_id="indo1319",
        parent_id="biha1245",
        name="Pali",
        level="language",
        iso639P3code="pli",
        type="a",
    ),
    "part1239": Language(
        family_id="indo1319",
        parent_id="nort3177",
        name="Parthian",
        level="language",
        iso639P3code="xpr",
        type="a",
    ),
    "pech1242": Language(
        family_id="turk1311",
        parent_id="unun9944",
        name="Pecheneg",
        level="language",
        iso639P3code="xpc",
        type="h",
    ),
    "phoe1239": Language(
        family_id="afro1255",
        parent_id="phoe1238",
        name="Phoenician",
        level="language",
        iso639P3code="phn",
        type="a",
    ),
    "phry1239": Language(
        family_id="indo1319",
        parent_id="grae1234",
        name="Phrygian",
        level="language",
        iso639P3code="xpg",
        type="a",
    ),
    "pict1238": Language(
        family_id="uncl1493",
        parent_id="uncl1493",
        name="Pictish",
        level="language",
        iso639P3code="xpi",
        type="h",
    ),
    "pisi1234": Language(
        family_id="indo1319",
        parent_id="uncl1514",
        name="Pisidian",
        level="language",
        iso639P3code="xps",
        type="a",
    ),
    "puni1241": Language(
        family_id="afro1255",
        parent_id="phoe1238",
        name="Punic",
        level="language",
        iso639P3code="xpu",
        type="a",
    ),
    "qata1238": Language(
        family_id="afro1255",
        parent_id="sayh1236",
        name="Qatabanian",
        level="language",
        iso639P3code="xqt",
        type="a",
    ),
    "raet1238": Language(
        family_id="uncl1493",
        parent_id="uncl1493",
        name="Raetic",
        level="language",
        iso639P3code="xrr",
        type="a",
    ),
    "saba1279": Language(
        family_id="afro1255",
        parent_id="sayh1236",
        name="Sabaic",
        level="language",
        iso639P3code="xsa",
        type="a",
    ),
    "sabi1245": Language(
        family_id="indo1319",
        parent_id="sabe1248",
        name="Sabine",
        level="dialect",
        iso639P3code="sbv",
        type="a",
    ),
    "sans1269": Language(
        family_id="indo1319",
        parent_id="indo1321",
        name="Sanskrit",
        level="language",
        iso639P3code="san",
        type="a",
    ),
    "saur1252": Language(
        family_id="indo1319",
        parent_id="indo1322",
        name="Sauraseni Prakrit",
        level="language",
        iso639P3code="psu",
        type="h",
    ),
    "sica1234": Language(
        family_id="uncl1493",
        parent_id="uncl1493",
        name="Sicana",
        level="language",
        iso639P3code="sxc",
        type="a",
    ),
    "sicu1234": Language(
        family_id="uncl1493",
        parent_id="uncl1493",
        name="Sicula",
        level="language",
        iso639P3code="scx",
        type="a",
    ),
    "side1240": Language(
        family_id="indo1319",
        parent_id="unun9888",
        name="Sidetic",
        level="language",
        iso639P3code="xsd",
        type="a",
    ),
    "sogd1245": Language(
        family_id="indo1319",
        parent_id="sogd1246",
        name="Sogdian",
        level="language",
        iso639P3code="sog",
        type="a",
    ),
    "sout2618": Language(
        family_id="indo1319",
        parent_id="sabe1248",
        name="South Picene",
        level="dialect",
        iso639P3code="spx",
        type="a",
    ),
    "sume1241": Language(
        family_id="",
        parent_id="",
        name="Sumerian",
        level="language",
        iso639P3code="sux",
        type="a",
    ),
    "tang1334": Language(
        family_id="sino1245",
        parent_id="qian1263",
        name="Tangut",
        level="language",
        iso639P3code="txg",
        type="a",
    ),
    "tart1237": Language(
        family_id="uncl1493",
        parent_id="uncl1493",
        name="Tartessian",
        level="language",
        iso639P3code="txr",
        type="a",
    ),
    "thra1250": Language(
        family_id="indo1319",
        parent_id="indo1319",
        name="Thracian",
        level="language",
        iso639P3code="txh",
        type="a",
    ),
    "tokh1242": Language(
        family_id="indo1319",
        parent_id="tokh1241",
        name="Tokharian A",
        level="language",
        iso639P3code="xto",
        type="a",
    ),
    "tokh1243": Language(
        family_id="indo1319",
        parent_id="tokh1241",
        name="Tokharian B",
        level="language",
        iso639P3code="txb",
        type="a",
    ),
    "tran1282": Language(
        family_id="indo1319",
        parent_id="tgbc1234",
        name="Transalpine Gaulish",
        level="language",
        iso639P3code="xtg",
        type="a",
    ),
    "tums1237": Language(
        family_id="indo1319",
        parent_id="saka1298",
        name="Tumshuqese",
        level="language",
        iso639P3code="xtq",
        type="h",
    ),
    "ugar1238": Language(
        family_id="afro1255",
        parent_id="nort3165",
        name="Ugaritic",
        level="language",
        iso639P3code="uga",
        type="a",
    ),
    "umbr1253": Language(
        family_id="indo1319",
        parent_id="sabe1249",
        name="Umbrian",
        level="language",
        iso639P3code="xum",
        type="a",
    ),
    "urar1245": Language(
        family_id="hurr1239",
        parent_id="hurr1239",
        name="Urartian",
        level="language",
        iso639P3code="xur",
        type="a",
    ),
    "vene1257": Language(
        family_id="indo1319",
        parent_id="indo1319",
        name="Venetic",
        level="language",
        iso639P3code="xve",
        type="a",
    ),
    "vols1237": Language(
        family_id="indo1319",
        parent_id="umbr1253",
        name="Volscian",
        level="dialect",
        iso639P3code="xvo",
        type="a",
    ),
    "zhan1239": Language(
        family_id="sino1245",
        parent_id="unun9961",
        name="Zhangzhung",
        level="language",
        iso639P3code="xzh",
        type="a",
    ),
    "arc": Language(
        family_id=None,
        parent_id=None,
        name="Official Aramaic (700-300 BCE)",
        level=None,
        iso639P3code="arc",
        type="a",
    ),
    "hmk": Language(
        family_id=None,
        parent_id=None,
        name="Maek",
        level=None,
        iso639P3code="hmk",
        type="a",
    ),
    "htx": Language(
        family_id=None,
        parent_id=None,
        name="Middle Hittite",
        level=None,
        iso639P3code="htx",
        type="a",
    ),
    "ims": Language(
        family_id=None,
        parent_id=None,
        name="Marsian",
        level=None,
        iso639P3code="ims",
        type="a",
    ),
    "lab": Language(
        family_id=None,
        parent_id=None,
        name="Linear A",
        level=None,
        iso639P3code="lab",
        type="a",
    ),
    "lng": Language(
        family_id=None,
        parent_id=None,
        name="Langobardic",
        level=None,
        iso639P3code="lng",
        type="a",
    ),
    "nei": Language(
        family_id=None,
        parent_id=None,
        name="Neo-Hittite",
        level=None,
        iso639P3code="nei",
        type="a",
    ),
    "oht": Language(
        family_id=None,
        parent_id=None,
        name="Old Hittite",
        level=None,
        iso639P3code="oht",
        type="a",
    ),
    "pgl": Language(
        family_id=None,
        parent_id=None,
        name="Primitive Irish",
        level=None,
        iso639P3code="pgl",
        type="a",
    ),
    "pgn": Language(
        family_id=None,
        parent_id=None,
        name="Paelignian",
        level=None,
        iso639P3code="pgn",
        type="a",
    ),
    "sxo": Language(
        family_id=None,
        parent_id=None,
        name="Sorothaptic",
        level=None,
        iso639P3code="sxo",
        type="a",
    ),
    "umc": Language(
        family_id=None,
        parent_id=None,
        name="Marrucinian",
        level=None,
        iso639P3code="umc",
        type="a",
    ),
    "xae": Language(
        family_id=None,
        parent_id=None,
        name="Aequian",
        level=None,
        iso639P3code="xae",
        type="a",
    ),
    "xaq": Language(
        family_id=None,
        parent_id=None,
        name="Aquitanian",
        level=None,
        iso639P3code="xaq",
        type="a",
    ),
    "xep": Language(
        family_id=None,
        parent_id=None,
        name="Epi-Olmec",
        level=None,
        iso639P3code="xep",
        type="a",
    ),
    "xha": Language(
        family_id=None,
        parent_id=None,
        name="Harami",
        level=None,
        iso639P3code="xha",
        type="a",
    ),
    "xhc": Language(
        family_id=None,
        parent_id=None,
        name="Hunnic",
        level=None,
        iso639P3code="xhc",
        type="a",
    ),
    "xli": Language(
        family_id=None,
        parent_id=None,
        name="Liburnian",
        level=None,
        iso639P3code="xli",
        type="a",
    ),
    "xln": Language(
        family_id=None,
        parent_id=None,
        name="Alanic",
        level=None,
        iso639P3code="xln",
        type="a",
    ),
    "xme": Language(
        family_id=None,
        parent_id=None,
        name="Median",
        level=None,
        iso639P3code="xme",
        type="a",
    ),
    "xmk": Language(
        family_id=None,
        parent_id=None,
        name="Ancient Macedonian",
        level=None,
        iso639P3code="xmk",
        type="a",
    ),
    "xpp": Language(
        family_id=None,
        parent_id=None,
        name="Puyo-Paekche",
        level=None,
        iso639P3code="xpp",
        type="a",
    ),
    "xpy": Language(
        family_id=None,
        parent_id=None,
        name="Puyo",
        level=None,
        iso639P3code="xpy",
        type="a",
    ),
    "xrm": Language(
        family_id=None,
        parent_id=None,
        name="Armazic",
        level=None,
        iso639P3code="xrm",
        type="a",
    ),
    "xsc": Language(
        family_id=None,
        parent_id=None,
        name="Scythian",
        level=None,
        iso639P3code="xsc",
        type="a",
    ),
    "xtr": Language(
        family_id=None,
        parent_id=None,
        name="Early Tripuri",
        level=None,
        iso639P3code="xtr",
        type="a",
    ),
    "xvn": Language(
        family_id=None,
        parent_id=None,
        name="Vandalic",
        level=None,
        iso639P3code="xvn",
        type="a",
    ),
    "xvs": Language(
        family_id=None,
        parent_id=None,
        name="Vestinian",
        level=None,
        iso639P3code="xvs",
        type="a",
    ),
    "zra": Language(
        family_id=None,
        parent_id=None,
        name="Kara (Korea)",
        level=None,
        iso639P3code="zra",
        type="a",
    ),
    "zsk": Language(
        family_id=None,
        parent_id=None,
        name="Kaskean",
        level=None,
        iso639P3code="zsk",
        type="a",
    ),
    "axm": Language(
        family_id=None,
        parent_id=None,
        name="Middle Armenian",
        level=None,
        iso639P3code="axm",
        type="h",
    ),
    "cmg": Language(
        family_id=None,
        parent_id=None,
        name="Classical Mongolian",
        level=None,
        iso639P3code="cmg",
        type="h",
    ),
    "cnx": Language(
        family_id=None,
        parent_id=None,
        name="Middle Cornish",
        level=None,
        iso639P3code="cnx",
        type="h",
    ),
    "ndf": Language(
        family_id=None,
        parent_id=None,
        name="Nadruvian",
        level=None,
        iso639P3code="ndf",
        type="h",
    ),
    "nwc": Language(
        family_id=None,
        parent_id=None,
        name="Classical Newari",
        level=None,
        iso639P3code="nwc",
        type="h",
    ),
    "nwx": Language(
        family_id=None,
        parent_id=None,
        name="Middle Newar",
        level=None,
        iso639P3code="nwx",
        type="h",
    ),
    "oav": Language(
        family_id=None,
        parent_id=None,
        name="Old Avar",
        level=None,
        iso639P3code="oav",
        type="h",
    ),
    "obt": Language(
        family_id=None,
        parent_id=None,
        name="Old Breton",
        level=None,
        iso639P3code="obt",
        type="h",
    ),
    "ohu": Language(
        family_id=None,
        parent_id=None,
        name="Old Hungarian",
        level=None,
        iso639P3code="ohu",
        type="h",
    ),
    "okm": Language(
        family_id=None,
        parent_id=None,
        name="Middle Korean (10th-16th cent.)",
        level=None,
        iso639P3code="okm",
        type="h",
    ),
    "oko": Language(
        family_id=None,
        parent_id=None,
        name="Old Korean (3rd-9th cent.)",
        level=None,
        iso639P3code="oko",
        type="h",
    ),
    "omp": Language(
        family_id=None,
        parent_id=None,
        name="Old Manipuri",
        level=None,
        iso639P3code="omp",
        type="h",
    ),
    "ota": Language(
        family_id=None,
        parent_id=None,
        name="Ottoman Turkish (1500-1928)",
        level=None,
        iso639P3code="ota",
        type="h",
    ),
    "otb": Language(
        family_id=None,
        parent_id=None,
        name="Old Tibetan",
        level=None,
        iso639P3code="otb",
        type="h",
    ),
    "pgd": Language(
        family_id=None,
        parent_id=None,
        name="Gāndhārī",
        level=None,
        iso639P3code="pgd",
        type="h",
    ),
    "pka": Language(
        family_id=None,
        parent_id=None,
        name="Ardhamāgadhī Prākrit",
        level=None,
        iso639P3code="pka",
        type="h",
    ),
    "sqr": Language(
        family_id=None,
        parent_id=None,
        name="Siculo Arabic",
        level=None,
        iso639P3code="sqr",
        type="h",
    ),
    "svx": Language(
        family_id=None,
        parent_id=None,
        name="Skalvian",
        level=None,
        iso639P3code="svx",
        type="h",
    ),
    "wlm": Language(
        family_id=None,
        parent_id=None,
        name="Middle Welsh",
        level=None,
        iso639P3code="wlm",
        type="h",
    ),
    "xbm": Language(
        family_id=None,
        parent_id=None,
        name="Middle Breton",
        level=None,
        iso639P3code="xbm",
        type="h",
    ),
    "xcb": Language(
        family_id=None,
        parent_id=None,
        name="Cumbric",
        level=None,
        iso639P3code="xcb",
        type="h",
    ),
    "xcu": Language(
        family_id=None,
        parent_id=None,
        name="Curonian",
        level=None,
        iso639P3code="xcu",
        type="h",
    ),
    "xgl": Language(
        family_id=None,
        parent_id=None,
        name="Galindan",
        level=None,
        iso639P3code="xgl",
        type="h",
    ),
    "xmn": Language(
        family_id=None,
        parent_id=None,
        name="Manichaean Middle Persian",
        level=None,
        iso639P3code="xmn",
        type="h",
    ),
    "xqa": Language(
        family_id=None,
        parent_id=None,
        name="Karakhanid",
        level=None,
        iso639P3code="xqa",
        type="h",
    ),
    "xzp": Language(
        family_id=None,
        parent_id=None,
        name="Ancient Zapotec",
        level=None,
        iso639P3code="xzp",
        type="h",
    ),
    "zkh": Language(
        family_id=None,
        parent_id=None,
        name="Khorezmian",
        level=None,
        iso639P3code="zkh",
        type="h",
    ),
    "zkz": Language(
        family_id=None,
        parent_id=None,
        name="Khazar",
        level=None,
        iso639P3code="zkz",
        type="h",
    ),
}


def make_iso_glotto_map():
    """Generate above dict of ISO code to Glottolog data. Writes
    file to ``~/Downloads/cltk_langs.dict``.
    """
    # Get ISO langs marked historical or ancient
    iso_fp = os.path.expanduser(
        "~/Downloads/iso-639-3_Code_Tables_20190408/iso-639-3_20190408.tab"
    )
    iso_name_h = dict()  # is historical type
    iso_name_a = dict()  # is ancient type
    with open(iso_fp) as file_open:
        csv_reader = csv.reader(file_open, delimiter="\t")
        for row in csv_reader:
            if row[5] == "H":
                code = row[0]
                name = row[6]
                iso_name_h[code] = name
            if row[5] == "A":
                code = row[0]
                name = row[6]
                iso_name_a[code] = name
    print("ancient", len(iso_name_a))
    print("historical:", len(iso_name_h))
    print("total iso:", len(iso_name_a) + len(iso_name_h))

    # Get Glottolog langs that have been marked as ancient or historical in ISO
    glottolog_csv = os.path.expanduser(
        "~/Downloads/glottolog_languoid.csv/languoid.csv"
    )
    # ['id,family_id,parent_id,name,bookkeeping,level,latitude,
    # longitude,iso639P3code,description,markup_description,
    # child_family_count,child_language_count,child_dialect_count,
    # country_ids']

    glotto_dict = defaultdict(Language)
    with open(glottolog_csv) as file_open:
        csv_reader = csv.reader(file_open, delimiter=",")
        for row in csv_reader:
            if row[8] in iso_name_a.keys():
                lang_type = "a"
                del iso_name_a[row[8]]
            elif row[8] in iso_name_h.keys():
                lang_type = "h"
                del iso_name_h[row[8]]
            else:
                continue
            glotto_lang = Language(
                family_id=row[1],
                parent_id=row[2],
                name=row[3],
                level=row[5],
                iso639P3code=row[8],
                type=lang_type,
            )
            glotto_dict[row[0]] = glotto_lang

    # Add those in ISO but not in Language
    for key, val in iso_name_a.items():
        glotto_lang = Language(
            family_id=None,
            parent_id=None,
            name=val,
            level=None,
            iso639P3code=key,
            type="a",
        )
        glotto_dict[key] = glotto_lang
    for key, val in iso_name_h.items():
        glotto_lang = Language(
            family_id=None,
            parent_id=None,
            name=val,
            level=None,
            iso639P3code=key,
            type="h",
        )
        glotto_dict[key] = glotto_lang
    print("total in glotto:", len(glotto_dict))
    with open(os.path.expanduser("~/Downloads/cltk_langs.dict"), "w") as file_open:
        file_open.write(str(glotto_dict))


if __name__ == "__main__":
    make_iso_glotto_map()  # pragma: no cover
